﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace ExceptionPOC
{
	/// <summary>
	/// A structure to layout data for printing.
	/// </summary>
	public class PrintLayout
	{
		#region Constants

		/// <summary>
		/// The names of properties and action relating to this class.
		/// </summary>
		public static partial class AttributeNames
		{
			/// <summary>
			/// The name of the Layouts property.
			/// </summary>
			public const string Layouts = "Layouts";
			
			/// <summary>
			/// The name of the Records property.
			/// </summary>
			public const string Records = "Records";
			
			/// <summary>
			/// The name of the Title property.
			/// </summary>
			public const string Title = "Title";


			/// <summary>
			/// The name of the SortOrder property.
			/// </summary>
			public const string SortOrder = "SortOrder";
		}

		/// <summary>
		/// The business logic settings and values.
		/// </summary>
		public static class BusinessLogic
		{
			/// <summary>
			/// The value for the MinimumSortOrder property.
			/// </summary>
			public const int MinimumSortOrder = 1;
		}

		#endregion Constants

		#region Constructors

		/// <summary>
		/// Constructor.
		/// </summary>
		/// <param name="title">The title of this data.</param>
		public PrintLayout(string title)
		{
			if (title.IsNull())
			{
				throw new ArgumentException("The title argument may not be Null.");
			}

			this.Title = title;
		}

		/// <summary>
		/// Constructor.
		/// </summary>
		/// <param name="title">The title of this data.</param>
		/// <param name="sortOrder">The sort order of this data.</param>
		public PrintLayout(string title, int? sortOrder)
		{
			if (title.IsNull())
			{
				throw new ArgumentException("The title argument may not be Null.");
			}

			this.Title = title;
			this.SortOrder = sortOrder;
		}

		#endregion Constructors

		#region Properties

		//// Note:  Keep the order of these properties, as this determines how they display.

		/// <summary>
		/// The title of this data.
		/// </summary>
		public string Title
		{
			get;
			set;
		}

		/// <summary>
		/// A list of records to print.
		/// </summary>
		public IList<PrintRecord> Records
		{
			get;
			set;
		}

		/// <summary>
		/// A list of layouts.
		/// </summary>
		public IList<PrintLayout> Layouts
		{
			get;
			set;
		}

		/// <summary>
		/// The sort order of this data.
		/// </summary>
		public int? SortOrder
		{
			get;
			set;
		}

		#endregion Properties

		/// <summary>
		/// Add a new layout to the list of layouts.
		/// </summary>
		/// <param name="newLayout">The new layout to add to the list.</param>
		/// <returns>False indicates an error occurred.</returns>
		public bool AddLayout(PrintLayout newLayout)
		{
			if (newLayout == null)
			{
				return false;
			}

			if (this.Layouts == null)
			{
				this.Layouts = new List<PrintLayout>();
			}

			// validate sort order
			if (!newLayout.SortOrder.HasValue)
			{
				newLayout.SortOrder = this.GetNextLayoutSortOrder();
			}

			this.Layouts.Add(newLayout);

			return true;
		}

		/// <summary>
		/// Add a new layout to the list of layouts.
		/// </summary>
		/// <param name="title">The title of this data.</param>
		/// <returns>
		/// The new layout object.
		/// Null indicates an error occurred.
		/// </returns>
		public PrintLayout AddLayout(string title)
		{
			if (title.IsNullOrEmpty())
			{
				return null;
			}

			PrintLayout newLayout = new PrintLayout(title);

			if (!this.AddLayout(newLayout))
			{
				return null;
			}

			return newLayout;
		}

		/// <summary>
		/// Add a new record to the list of records.
		/// </summary>
		/// <param name="newRecord">The new record to add to the list.</param>
		/// <returns>False indicates an error occurred.</returns>
		public bool AddRecord(PrintRecord newRecord)
		{
			if (newRecord == null)
			{
				return false;
			}

			if (this.Records == null)
			{
				this.Records = new List<PrintRecord>();
			}

			// validate sort order
			if (!newRecord.SortOrder.HasValue)
			{
				newRecord.SortOrder = this.GetNextRecordSortOrder();
			}

			this.Records.Add(newRecord);

			return true;
		}

		/// <summary>
		/// Add a new record to the list of records.
		/// </summary>
		/// <param name="label">The label of this data.</param>
		/// <param name="data">The value of this data.</param>
		/// <returns>
		/// The new record object.
		/// Null indicates an error occurred.
		/// </returns>
		public PrintRecord AddRecord(string label, string data)
		{
			if (label.IsNullOrEmpty() 
				&& data.IsNullOrEmpty())
			{
				return null;
			}

			PrintRecord newRecord = new PrintRecord(label, data);

			if (!this.AddRecord(newRecord))
			{
				return null;
			}

			return newRecord;
		}

		/// <summary>
		/// Get the next Layout sort order number that should be used.
		/// </summary>
		/// <returns>The next Layout sort order number that should be used.</returns>
		private int? GetNextLayoutSortOrder()
		{
			if (this.Layouts.Count > 0)
			{
				return this.Layouts.Max(sortCheck => sortCheck.SortOrder) + 1;
			}
			else
			{
				return BusinessLogic.MinimumSortOrder;
			}
		}

		/// <summary>
		/// Get the next Record sort order number that should be used.
		/// </summary>
		/// <returns>The next Record sort order number that should be used.</returns>
		private int? GetNextRecordSortOrder()
		{
			if (this.Records.Count > 0)
			{
				return this.Records.Max(sortCheck => sortCheck.SortOrder) + 1;
			}
			else
			{
				return PrintRecord.BusinessLogic.MinimumSortOrder;
			}
		}
	}
}
