﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace ExceptionPOC
{
	/// <summary>
	/// A record to contain data for printing.
	/// </summary>
	public class PrintRecord
	{
		#region Constants

		/// <summary>
		/// The names of properties and action relating to this class.
		/// </summary>
		public static class AttributeNames
		{
			/// <summary>
			/// The name of the Data property.
			/// </summary>
			public const string Data = "Data";
			
			/// <summary>
			/// The name of the Label property.
			/// </summary>
			public const string Label = "Label";
			
			/// <summary>
			/// The name of the Layouts property.
			/// </summary>
			public const string Layouts = "Layouts";

			/// <summary>
			/// The name of the SortOrder property.
			/// </summary>
			public const string SortOrder = "SortOrder";
		}

		/// <summary>
		/// The business logic settings and values.
		/// </summary>
		public static class BusinessLogic
		{
			/// <summary>
			/// The value for the MinimumSortOrder property.
			/// </summary>
			public const int MinimumSortOrder = 1;
		}

		#endregion Constants

		#region Constructors

		/// <summary>
		/// Constructor.
		/// </summary>
		/// <param name="label">The label of this data.</param>
		/// <param name="data">The value of this data.</param>
		public PrintRecord(string label, string data)
		{
			if (label.IsNull()
				&& data.IsNull())
			{
				throw new ArgumentException("Both arguments may not be Null.");
			}

			this.Label = label;
			this.Data = data;
		}

		/// <summary>
		/// Constructor.
		/// </summary>
		/// <param name="label">The label of this data.</param>
		/// <param name="data">The value of this data.</param>
		/// <param name="sortOrder">The sort order of this data.</param>
		public PrintRecord(string label, string data, int? sortOrder)
		{
			if (label.IsNull()
				&& data.IsNull())
			{
				throw new ArgumentException("Both the Label and the Data arguments may not be Null.");
			}

			this.Label = label;
			this.Data = data;
			this.SortOrder = sortOrder;
		}

		#endregion Constructors

		#region Properties

		/// <summary>
		/// The value of this data.
		/// </summary>
		public string Data
		{
			get;
			set;
		}

		/// <summary>
		/// The label of this data.
		/// </summary>
		public string Label
		{
			get;
			set;
		}

		/// <summary>
		/// A list of layouts.
		/// </summary>
		public IList<PrintLayout> Layouts
		{
			get;
			set;
		}

		/// <summary>
		/// The sort order of this data.
		/// </summary>
		public int? SortOrder
		{
			get;
			set;
		}

		#endregion Properties

		/// <summary>
		/// Add a new layout to the list of layouts.
		/// </summary>
		/// <param name="newLayout">The new layout to add to the list.</param>
		/// <returns>False indicates an error occurred.</returns>
		public bool AddLayout(PrintLayout newLayout)
		{
			if (newLayout == null)
			{
				return false;
			}

			if (this.Layouts == null)
			{
				this.Layouts = new List<PrintLayout>();
			}

			// validate sort order
			if (!newLayout.SortOrder.HasValue)
			{
				newLayout.SortOrder = this.GetNextSortOrder();
			}

			this.Layouts.Add(newLayout);

			return true;
		}

		/// <summary>
		/// Add a new layout to the list of layouts.
		/// </summary>
		/// <param name="title">The title of this data.</param>
		/// <returns>
		/// The new layout object.
		/// Null indicates an error occurred.
		/// </returns>
		public PrintLayout AddLayout(string title)
		{
			if (title.IsNullOrEmpty())
			{
				return null;
			}

			PrintLayout newLayout = new PrintLayout(title);

			if (!this.AddLayout(newLayout))
			{
				return null;
			}

			return newLayout;
		}

		/// <summary>
		/// Get the next sort order number that should be used.
		/// </summary>
		/// <returns>The next sort order number that should be used.</returns>
		private int? GetNextSortOrder()
		{
			if (this.Layouts.Count > 0)
			{
				return this.Layouts.Max(sortCheck => sortCheck.SortOrder) + 1;
			}
			else
			{
				return PrintLayout.BusinessLogic.MinimumSortOrder;
			}
		}
	}
}
